module ClientPages
  module MarkdownContent
    class ContentBuilder
      attr_accessor :html

      def initialize(html)
        @html = html
      end

      def modify(tag, options={})
        html_lines do |line|
          conditions(tag, line, options[:if]) do
            match(tag, line, :open) { |open_tag| open_tag.gsub(/>/, "#{attributes(options)}>") }
          end
        end
      end

      def wrap(tag, wrapper, options={})
        html_lines do |line|
          conditions(tag, line, options[:if]) do
            line = match(tag, line, :open) { |rest| "<#{wrapper.to_s + attributes(options)}>" + rest }
            line = match(tag, line, :close) { |rest| rest + "</#{wrapper}>" }
          end
        end
      end

      def replace(tag, new_tag, options={})
        html_lines do |line|
          conditions(tag, line, options[:if]) do
            line = match(tag, line, :open) { |open_tag| "<#{new_tag.to_s + attributes(options)}>" }
            line = match(tag, line, :close) { |closing_tag| "</#{new_tag}>" }
          end
        end
      end

      private

      def match(tag, line, state)
        Matcher.new(tag, line).find(state) { |match| yield match }
      end

      def conditions(tag, line, condition)
        return yield if condition.nil?
        if condition.respond_to?(:call) && Matcher.new(tag, line).instance_exec(&condition)
          yield
        elsif condition.is_a?(Hash) && Matcher.new(tag, line).match_all?(condition)
          yield
        elsif condition == true
          yield
        else
          line
        end
      end

      def html_lines(&block)
        self.html = html.lines.map { |line| yield line }.join
      end

      def attributes(options)
        options.fetch(:with, {}).map { |k, v| " #{k}=\"#{v}\"" }.join
      end
    end
  end
end
