module ClientPages
  module MarkdownContent
    class Renderer
      class << self
        def render_content(file, options = {}, &block)
          html = new(file, options).html
          block_given? ? ContentBuilder.new(html).instance_eval(&block) : html.strip
        rescue OpenURI::HTTPError
          "404: could not find '#{file}'"
        rescue Errno::ENOENT
          "No such file or directory - #{file}"
        rescue SocketError
          'Error: content_path not available'
        end
      end

      attr_reader :file, :options

      def initialize(file, options = {})
        @file, @options = file, options
      end

      def html
        html = html_from_markdown
        html = template(html) if options.key?(:template)
        html
      end

      private

      def html_renderer
        Redcarpet::Render::HTML.new(config.render_options)
      end

      def extensions
        config.markdown_extensions.merge(options.key?(:extensions) ? options[:extensions] : {})
      end

      def html_from_markdown
        Redcarpet::Markdown.new(html_renderer, extensions).render(markdown)
      end

      def template(html)
        ContentBuilder.new(html).instance_eval(&Template[options[:template]])
      end

      def markdown
        raw? ? file : read_file
      end

      def url
        "#{content_path}/#{file}.md"
      end

      def read_file
        if caching?
          cache.fetch(url) { cache[url] = open(url).read }
        else
          open(url).read
        end
      end

      def content_path
        config.remote ? config.remote_content_path : config.content_path
      end

      def raw?
        config.raw_markdown || options[:raw]
      end

      def caching?
        config.caching
      end

      def config
        ClientPages.config
      end

      def cache
        ClientPages.cache
      end
    end
  end
end
